import os
import glob
import shutil


WIFI_CACHE = os.path.expanduser('~/.rpiup/wifi')


def add(ssid=None, password=None, path='.', default=False, country='US', remember=False):
    print('*** Setting up Wifi Credentials: ***')

    # check for saved wifi

    if not ssid and remember is not False and os.path.isfile(WIFI_CACHE):
        try:  # TODO: protect this file better
            with open(WIFI_CACHE, 'r') as f:
                ssid_cache = f.readline().strip()
                password_cache = f.readline().strip()

            use = input((
                'Would you like to use your previously saved credentials?'
                '\n  ssid: "{}" password: {}'
                '? ([y]/n) '
            ).format(ssid_cache, "*"*len(password_cache))).lower()  in {'y', ''}
            print()
            if use:
                remember, ssid, password = True, ssid_cache, password_cache
        except Exception as e:
            print(f'Error reading cached password. ({type(e).__name__}) {e}')

    # get ssid and password

    if not ssid:
        ssid = input('Enter the wifi ssid: ')
    if not ssid:
        print('No ssid specified.')
        return

    if ssid and not password:
        import getpass
        password = getpass.getpass('Enter the wifi password: ')

    # setup wifi file
    fname = os.path.join(path, 'resources/aps/{}.conf'.format(ssid))
    default_fname = os.path.join(path, 'wpa_supplicant.conf')

    if password:
        # custom ssid and password
        print('Creating wpa_supplicant file for ssid:', ssid)
        write_cfg(fname, ssid=ssid, psk=password, country=country)
    # elif os.path.isfile(fname):
    #     print('ssid exists:', fname)
    else:
        print('Creating unsecured network configuration for ssid:', ssid)
        write_cfg(fname, ssid=ssid, key_mgmt='NONE', country=country)

    if default or not os.path.isfile(default_fname):
        shutil.copyfile(fname, default_fname)

    # maybe save wifi

    if remember is None:
        remember = input(
            'Would you like to save the password for next time? ([y]/n) '
        ).lower() in {'y', ''}
        print()

    if remember is not False:
        os.makedirs(os.path.dirname(WIFI_CACHE), exist_ok=True)
        with open(WIFI_CACHE, 'w') as f:
            f.write(f'{ssid}\n')
            f.write(f'{password}\n')
        print(f'Credentials cached to {WIFI_CACHE}.')
    print()

    print('*** Done! :) ***')
    print()

    print('To check that your wifi credentials look correct, run:', '\n  ',
          f'less {fname}')


def aps_from(dir, app_path='.'):
    for f in glob.glob(os.path.join(dir, '*')):
        shutil.copyfile(f, os.path.join(app_path, 'resources', os.path.basename(f)))


def clear_cache():
    if os.path.isfile(WIFI_CACHE):
        os.remove(WIFI_CACHE)
    print('{} removed ({}).'.format(WIFI_CACHE, not os.path.isfile(WIFI_CACHE)))


def touch(fname, text=None, time=None):
    with open(fname, 'w') as f:
        os.utime(fname, time)
        if text:
            f.write(str(text))


def format_cfg(country='US', **kw):
    return WPA_SUPPLICANT_TPL.format(country=country, params='\n'.join([
        '\t{}="{}"'.format(k, v) for k, v in kw.items()
    ]))

def write_cfg(fname, **kw):
    with open(fname, 'w') as f:
        f.write(format_cfg(**kw))


WPA_SUPPLICANT_TPL = '''
ctrl_interface=/var/run/wpa_supplicant
update_config=1
country={country}

network={{
{params}
}}
'''.strip()

if __name__ == '__main__':
    import fire
    fire.Fire(add)



import os
import shlex
from . import util

VARS_FNAME = 'vars.sh'

def get_fname(path):
    return path if os.path.isfile(path) else os.path.join(path, VARS_FNAME)

RESERVED_NAMES = {
    'hostname': 'firstboot_hostname'
}


class Vars:
    DEFAULT_PATH = '.'
    def __init__(self, path='./resources', autosave=True, overwrite=False):
        self.fname = get_fname(path)
        self.config = {}
        self.current_config = {}
        self.autosave = autosave
        if not overwrite:
            self.load()

    def __str__(self):
        import yaml
        return '--- Variables in {}: \n{}---'.format(self.fname, yaml.dump(self.config))

    def load(self):
        '''Load existing variables from a string'''
        if os.path.isfile(self.fname):
            with open(self.fname, 'r') as f:
                self.current_config = self.parse(f.read())
                self.update(self.current_config, save_=False)
        return self

    def update(self, dct=(), save_=None, **kw):
        self.config.update(dct)
        self.config.update(kw)
        if self.autosave if save_ is None else save_:
            self.save()
        return self

    def pop(self, name, *a, **kw):
        return self.config.pop(name, *a, **kw)

    def save(self):
        changed_vars = [
            '{}={}'.format(k, v) for k, v in self.config.items()
            if self.current_config.get(k, ...) != v
        ]
        if not changed_vars:
            return
        print('*** Setting Variables ({}). ***'.format(', '.join(changed_vars)))
        util.touch(self.fname, '\n'.join([
            'export {}={}'.format(RESERVED_NAMES.get(k.lower(), k), shlex.quote(v)) for k, v in self.config.items() if v
        ]) + '\n')
        print()
        self.current_config = dict(self.config)
        return self

    def get(self, name, *a, **kw):
        return self.config.get(name, *a, **kw)

    def set(self, *args, **kw):
        return self.parse_string(*args, **kw) if args else self.prompt_any(**kw)

    def prompt(self, name, desc='', default=None, value=None, secret=False, update=True):
        '''Prompt user for a specific variable.'''
        if value is None:
            print('*'*20)
            default = self.config.get(name) or default
            while True:
                value = util.getinput('{} [{}] {}='.format(desc, default, name), secret=secret)
                if value or default is not None:
                    break
                print('{} is required.'.format(name))
            print()
        if update:
            self.update(**{name: value or default})
        return value or default

    def prompt_any(self, n=None, **kw):
        '''Prompt user for arbitrary KEY=VALUE pairs to store on the sd card.'''
        print('*** Enter variables to store (KEY=VALUE): ***')
        for i in util.count(n):
            inp = input('... ')
            if not inp:
                break
            parts = inp.split('=', 1)
            if len(parts) != 2:
                print('*** ERROR - Input {} must be of the format KEY=VALUE ***'.format(inp))
                continue
            kw[parts[0]] = parts[1]
        return self.update(kw)

    def parse_string(self, *args, **kw):
        '''Parse from a string in the form of KEYA=1,KEYB=2,KEY3=...'''
        return self.update(dict(
            kw, (l.split('=') for ls in args if ls for l in ls.split(','))))

    @staticmethod
    def parse(text, comments=False, **kw):
        cfg, cmts = {}, []
        for l in text.strip().splitlines():
            if l.strip().startswith('#'):
                cmts.append(l)
            else:
                k, v = l.split('=')
                cfg[k.split(' ')[-1]] = strip_quotes(v)
        cfg.update(kw)
        return (cfg, comments) if comments else cfg


def strip_quotes(x):
    if isinstance(x, str) and len(x) >= 2 and x[0] == x[1] and x[0] in '"\'':
        return x[1:-1]
    return x



import os
import functools
from . import varsfile
from . import util
from . import wifi


def ssh_file(path):
    print('*** Enabling SSH. ***')
    util.touch(os.path.join(path, 'ssh'))
    print()

DEFAULT_SD_PATH = '/Volumes/boot'

def setup(path='.', sd_path=DEFAULT_SD_PATH, uuid=None, hostname=None, **kw):
    '''You've just flashed a fresh raspbian verson.

    Now you want to:
     - enable ssh so you can access the pi without a keyboard/monitor.
     - setup your wifi network so you can access it via ssh.

    You can specify your own network, or it will default to `s0nycL1f3l1ne`.

    Arguments:
        ssid (str): the wifi ssid
        password (str):the wifi password
        sd_path (str): the path to the sd card. It should always be /Volumes/boot,
            but you never know.
        ssh (bool): whether to enable ssh. Idk why you'd ever want to disable,
            if you do, but use: `--nossh`

    '''
    try:
        if not os.path.exists(sd_path):
            print('*** ERROR - No SD Card found at {} ***'.format(sd_path))
            return

        print('Copying over boot files...')
        util.copytree(path, sd_path)
        print()

        var = varsfile.Vars(os.path.join(sd_path, 'resources'))
        var.update(kw)
        hostname_prefix = '{}node'.format(var.get('APP_NAME'))
        var.prompt('UUID', 'Do you want to set a deployment UUID? This can be used to tie status messages to specific hardware.', '', uuid)
        var.prompt('hostname', 'Do you want to set the device hostname? (default: {}-$MAC_ADDR)'.format(hostname_prefix), '', hostname)
        if hostname is False or not var.get('hostname'):
            var.prompt('hostname_prefix', 'Do you want to change the device hostname prefix? (default: {})'.format(hostname_prefix), '')
        var.prompt_any()

        print('Please eject the SD card, insert it into your Pi, and start up!')
    except KeyboardInterrupt:
        print('Interrupted.')


# @functools.wraps(varsfile.Vars)
def vars(path=os.path.join(DEFAULT_SD_PATH, 'resources'), *a, **kw):
    return varsfile.Vars(path, *a, **kw)


def check(sd_path=DEFAULT_SD_PATH, uuid=None):
    var = varsfile.Vars(os.path.join(sd_path, 'resources'))
    if uuid:
        assert 'uuid' in var.config, 'missing UUID. Set using rpiup sd vars set --UUID=something-asdfasdf'



import os
import flask
import yaml
import datetime
# import filelock
import sqlitedict


PORT = 8056


app = flask.Flask(__name__, template_folder='templates')#os.path.join(os.path.dirname(__file__), )


DEVICES = sqlitedict.SqliteDict('devices.db', autocommit=True)


'''

Accounting

'''


@app.route('/update/<device>/<status>')
def update(device, status=None):
    # use flask oidc and check for provisioner role
    d = DEVICES.get(device) or {'steps': [], 'last_updated': None, 'attrs':{},}
    args = dict(flask.request.args)

    t_now = datetime.datetime.now()
    statuses = []
    if status and status not in set(s['name'] for s in d['steps']):
        status, = statuses = [{'name': status, 'time': t_now}]
        err = args.pop('error', None)
        if err:
            status['error'] = err

    DEVICES[device] = {
        'steps': d['steps'] + statuses,
        'last_updated': t_now,
        'attrs': dict(d['attrs'], **flask.request.args),
    }
    if flask.request.args.get('ip'):
        refresh_inventory()
    # print(DEVICES[device])
    return flask.jsonify({'message': 'success'})


@app.route('/remove/<device>')
def remove(device):
    d = DEVICES[device].pop(device, None)
    refresh_inventory()
    return flask.jsonify({'message': 'success'})


@app.route('/clear')
def clear():
    DEVICES.clear()
    refresh_inventory()
    return flask.jsonify({'message': 'success'})

# export

@app.route('/inventory')
def inventory():
    with open('./inventory', 'r') as f:
        return '<pre>{}</pre>'.format(f.read())


@app.route('/inventory/refresh')
def inventoryrefresh():
    refresh_inventory()
    return flask.redirect(flask.url_for('inventory'))



'''

Status

'''



@app.route('/status')
def status():
    return flask.jsonify({'devices': dict(DEVICES)})


@app.route('/status/yaml')
def statusyaml():
    return flask.jsonify({'body': '<pre>{}</pre>'.format(yaml.dump(dict(DEVICES)))})


@app.route('/status/text')
def statustext():
    done, running = split_cond(DEVICES.items(), lambda x: any(x.endswith('done') for x in x[1]['steps']))

    return flask.jsonify({'body': '''{}'''.format(
        '\n'.join(
            [devicehtml(name, **d) for name, d in sorted(running)] +
            [tag('h4', 'Done ({})'.format(len(done))) if done else ''] +
            [devicehtml(name, done=True, **d) for name, d in sorted(done)]
        ).strip() or tag('h4', 'No Devices')
    )})


_ATTR_ALTS = {'c': 'class'}
_as_listitems = lambda xs: ' '.join(str(x) for x in xs if x) if isinstance(xs, (list, tuple)) else xs
tag = lambda _tag, *content, **kw: '<{tag} {attrs}>{content}</{tag}>'.format(
    tag=_tag, content=''.join(str(c) for c in content if c),
    attrs=' '.join(
        '{}="{}"'.format(_ATTR_ALTS.get(k, k), _as_listitems(v))
        for k, v in kw.items() if v is not None
    ))

tagmap = lambda _tag, *content, **kw: ''.join(tag(_tag, c, **kw) for c in content if c)
pill = lambda x, name=None: tag('span', name, name and ':', x) if x else ''


def devicehtml(name, steps, last_updated=None, done=False, attrs=None):
    return '<div class="device {}">{}</div>'.format(
        'done' if done else '',
        ''.join(tag('div', l) for l in [
            tag('span', name) + pill(attrs.get('ip'), 'ip'),
            tag('span', last_updated),
            ' | '.join(tag('span', s) for s in steps[::-1]),
            ''.join(pill(v, k) for k, v in attrs.items() if k != 'ip'),
        ]))


def split_cond(xs, cond):
    groups = {}
    for x in xs:
        groups.setdefault(cond(x), []).append(x)
    return [groups.get(k, []) for k in (True, False)] if not set(groups) - set((True, False)) else groups



'''

Pages

'''



@app.route('/watch')
def watch():
    # the most basic we can get
    return flask.render_template('index.html')

@app.route('/')
def index():
    return flask.jsonify({'message': "Welcome! Let's set up some devices!"})


import traceback
@app.errorhandler(Exception)
def handle_exception(e):
    return flask.jsonify({
        'error': True,
        'message': '''
{err}<pre><h3>Traceback (most recent call last):</h3><div>{tb}</div><h3>{typename}: {err}</h3></pre>
        '''.strip().format(
            err=e, tb='<hr/>'.join(traceback.format_tb(e.__traceback__)).strip('\n'),
            typename=type(e).__name__)
    }), 500


'''

Utils

'''

def refresh_inventory(fname='./inventory'):
    # with filelock.FileLock(fname + ".lock"):
    with open(fname, "w") as f:
        f.write(os.linesep.join(ip for ip in (
            d['attrs'].get('ip') for d in DEVICES.values()
        ) if ip))


if __name__=='__main__':
    app.run(host='0.0.0.0', port=PORT, debug=True)



import os
import re
import yaml
from . import util


def url(version=None, official=False, repo='nmcclain/raspberian-firstboot'):
    if official:
        if version:
            # https://downloads.raspberrypi.org/raspios_lite_armhf/images/raspios_lite_armhf-2020-12-04/2020-12-02-raspios-buster-armhf-lite.zip
            download_url = 'https://downloads.raspberrypi.org/raspios_lite_armhf/images/{}.zip'.format(version)
        else:
            download_url = 'https://downloads.raspberrypi.org/raspios_lite_armhf_latest'
            version = get_file_download_name(url)
    else:
        import urllib.request
        if version:
            download_url = 'https://s3.amazonaws.com/devicecfg.newmoon.io/images/raspbian-lite-firstboot/{}.zip'.format(version)
        else:
            url = 'https://api.github.com/repos/{}/releases/latest'.format(repo)
            with urllib.request.urlopen(url) as response:
                pkg_info = yaml.safe_load(response)
            print('*** Found latest: ***')
            print(pkg_info['body'])
            print('*' * 20, '\n')

            matches = re.search(r'\[\w*\s*([^\]]+)\.img]\(([^\)]+)\)', pkg_info['body'])
            version, download_url = matches.groups()
    return download_url, version

def download(output_path='.', *a, **kw):
    '''Download the latest version.'''
    # get download url
    download_url, version = url(*a, **kw)
    print('Downloading Release from {}'.format(download_url))
    util.download_file(download_url, output_path, name='{}.zip'.format(version))


# def flash(img_fname):
#     pass



''''''
# generate certificate
# generate wifi
import os as os_
from . import util
from . import os
from . import sd
from . import varsfile
from . import wifi
from .util import DEFAULT_SRC


DEFAULT_PATH = '.'

def init(name=None, path=None, ssid=None, psk=None, user=None, password=None,
         git_user=None, git_password=None, src=None, ssh=True, boot_script='full-docker'):
    '''Copy over files. Interactive setup?'''
    path = path or name or DEFAULT_PATH
    print('''
****************
rpiup: Initializing Project.

Copying sd boot files to: {path}

After the initial setup, (this command), you can edit the files inside "{path}".

 - setup.sh: this is a script to run arbitrary setup. This is run after the system is setup (Docker is available)
 - vars.sh: this is used to store arbitrary variables that are needed during setup. this can include any
            certificates or passwords.
 - aps.yml: this contains the wpa supplicant information for any wifi networks you want to connect to.
 - services/: where you can store arbitrary services that you want to be installed on first boot.
    - servicename/: each service is stored in its own subdirectory
        - install.sh: used to do any installs, wget, apt-get, cp, mkdir, etc. that you need.
        - servicename.service: this is a systemctl service. ** installation of this file will be handled for you. **
 - docker/: any docker containers that you want to start
    - servicename:
        - docker-compose.yml: the docker-compose file that describes your container.

Now that you've finished the general setup, insert your SD card and flash it using Balena Etcher.


When that's finished run:
```
rpiup sd setup {maybe_path}
```

And then eject the SD card and put it in your pi!
****************

    '''.format(path=path, maybe_path=path if path != DEFAULT_PATH else ''))

    try:
        util.copytree(DEFAULT_SRC, path, overwrite=False)
        if src:
            util.copytree(src, path, overwrite=True)

        # set app name
        name = name or os_.path.basename(os_.path.abspath(path)) or ''
        var = varsfile.Vars(os_.path.join(path, 'resources'))
        if len(var.config):
            print('current variables:', set(var.config), '\n')
        var.prompt('APP_NAME', 'What is the app name?', name)
        var.prompt('USERNAME', 'What should the device username be?', name, user)
        var.prompt('PASSWORD', 'What should the device password be?', '', password, secret=True)

        var.prompt('GIT_USERNAME', 'Do you want to set a git username?', '', git_user)
        var.prompt('GIT_PASSWORD', 'Do you want to set a git password?', '', git_password, secret=True)

        ip = util.localip()
        if ip:
            host = '{}:{}'.format(ip, 8056)
            # if set(util.check_json_response('http://{}'.format(host)) or ()) == {'message'}:
            var.update(MONITOR_HOST=host)

        if ssh:  # set ssh
            sd.ssh_file(path)

        # add default wifi
        wifi.add(ssid, psk, path=path, remember=None)

        ap_path = var.prompt(
            'AP_PATH', 'Do you have a set of alternate wifi credentials you would like to add?\n  (Read more at https://github.com/beasteers/netswitch) \nWhat is their path?', '', update=False)
        if ap_path:
            util.copytree(ap_path, os_.path.join(path, 'resources/aps/'))

        var.prompt('LIFELINE_SSID', 'Do you have a wifi lifeline?\nNOTE: This is a wifi network that your device will always connect to with the highest priority. This is useful for engineers debugging a sensor in the field.\nWhat is the SSID?', '')

    except KeyboardInterrupt:
        print('Interrupted')


def describe(path=None):
    path = os_.path.abspath(path or DEFAULT_PATH)
    print('**', path, '**')
    util.file_tree(path, ignore={'.DS_Store'})
    print('*'*10, '\n')
    _file_summary(os_.path.join(path, 'resources/vars.sh'))
    _file_summary(os_.path.join(path, 'wpa_supplicant.conf'))


def _file_summary(path):
    print('**', path, '**')
    print(util.file_content(path).strip())
    print('*'*10)
    print()




def cli():
    import fire
    fire.Fire({
        'init': init, 'os': os, 'sd': sd, 'vars': varsfile.Vars,
        'wifi': wifi, 'copydiff': util.copydiff, 'util': util,
        'describe': describe
    })



import os
import re
import subprocess


DEFAULT_SRC = os.path.abspath(os.path.join(os.path.dirname(__file__), 'boot-files'))


def count(start=0, end=None, step=1):
    i = start or 0
    while True:
        yield i
        i += step
        if end and i >= end:
            break

def touch(fname, text=None, time=None):
    with open(fname, 'w') as f:
        os.utime(fname, time)
        if text:
            f.write(str(text))


def getinput(msg='', secret=False):
    if secret:
        import getpass
        return getpass.getpass(msg)
    return input(msg)


def download_file(url, output_path='.', name=None, overwrite=False):
    import urllib.request

    filename = None
    if output_path:
        output_path = os.path.expanduser(output_path)
        os.makedirs(output_path, exist_ok=True)
        filename = os.path.join(output_path, name or get_file_download_name(url))

        # check if the file already exists
        if not overwrite and os.path.isfile(filename):
            redownload = input(
                'It looks like this image is already downloaded at {}.'
                'Do you wish to redownload? y[N]  '.format(filename)
            ).lower().startswith('y')
            if not redownload:
                print('Okie! Not re-downloading. Have a nice day!')
                return
        print('Writing to {}'.format(filename))

    import tqdm
    class DownloadProgressBar(tqdm.tqdm):
        def update_to(self, b=1, bsize=1, tsize=None):
            if tsize is not None:
                self.total = tsize
            self.update(b * bsize - self.n)

    with DownloadProgressBar(unit='B', unit_scale=True, miniters=1,
                             desc=filename or url.split('/')[-1]) as t:
        urllib.request.urlretrieve(url, filename=filename, reporthook=t.update_to)


def get_file_download_name(url):
    import urllib.request
    import cgi
    header = urllib.request.urlopen(url).info()['Content-Disposition']
    return cgi.parse_header(header)[0]["filename"]



def check_json_response(url):
    import json
    import urllib.request
    try:
        with urllib.request.urlopen(url) as response:
            return json.loads(response.read())
    except urllib.error.URLError:
        pass
    return


def localip(n=1):
    out = subprocess.check_output('ifconfig').decode()
    match = set(re.findall(r'inet ([\d.]+)', out))
    return max((
        (i, ip) for i, ip in
        ((max_common_prefix('192.168.1'.split('.'), ip.split('.')), ip)
         for ip in match)
        if i >= n
    ))[1]


def max_common_prefix(*strs):
    return next(
        (i for i, chs in enumerate(zip(*strs)) if len(set(chs)) > 1),
        min(len(s) for s in strs))



def copytree(src, dst, symlinks=False, ignore=None, overwrite=None):
    import shutil
    if not os.path.exists(dst):
        os.makedirs(dst)
        shutil.copystat(src, dst)
    for item in listdir(src, ignore=ignore):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if symlinks and os.path.islink(s):
            _copyslink(s, d)
        elif os.path.isdir(s):
            copytree(s, d, symlinks, ignore, overwrite=overwrite)
        elif (overwrite is True or not os.path.exists(d) or
              not overwrite is False and os.stat(s).st_mtime - os.stat(d).st_mtime > 1):
            shutil.copy(s, d)

def _copyslink(s, d):
    import stat
    if os.path.lexists(d):
        os.remove(d)
    os.symlink(os.readlink(s), d)
    try:
        os.lchmod(d, stat.S_IMODE(os.lstat(s).st_mode))
    except Exception:
        pass # lchmod not available

def listdir(src, ignore=None):
    lst = os.listdir(src)
    if ignore:
        excl = set(ignore(src, lst))
        lst = [x for x in lst if x not in excl]
    return lst

def file_tree(path, ignore=None):
    for root, dirs, files in os.walk(path):
        level = root.replace(path, '').count(os.sep)
        print('{}{}/'.format(' ' * 4 * (level), os.path.basename(root)))
        for f in files:
            if ignore and f in ignore:
                continue
            print('{}{}'.format(' ' * 4 * (level + 1), f))

def file_content(path):
    with open(path, 'r') as f:
        return f.read()


def copydiff(output_path='./diff', path='.', ref=DEFAULT_SRC, test=False):
    '''Copy only files that are different.'''
    import shutil
    import filecmp
    filecmp.DEFAULT_IGNORES.append('.DS_Store')

    class dircmp2(filecmp.dircmp):
        def phase3(self): # Find out differences between common files
            self.same_files, self.diff_files, self.funny_files = filecmp.cmpfiles(
                self.left, self.right, self.common_files, shallow=False)

    def get_diff_files(dcmp):
        for name in dcmp.diff_files:
            print("diff_file %s found in %s and %s" % (name, dcmp.left, dcmp.right))
            yield os.path.join(dcmp.left, name)

        for name in dcmp.left_only:
            print("left_only %s found in %s but not %s" % (name, dcmp.left, dcmp.right))
            yield os.path.join(dcmp.left, name)

        for sub_dcmp in dcmp.subdirs.values():
            yield from get_diff_files(sub_dcmp)

    # move files
    ignore = filecmp.DEFAULT_IGNORES + [os.path.abspath(p) for p in (output_path, path, ref)] + ['ssh', 'wpa_supplicant.conf']
    for f in get_diff_files(dircmp2(path, ref, ignore=ignore)):
        if not os.path.isdir(f):
            outf = os.path.join(output_path, os.path.relpath(f, path))
            print('copying', f, 'to', outf)
            if not test:
                os.makedirs(os.path.dirname(outf), exist_ok=True)
                shutil.copy2(f, outf)
        print()



import os
import glob
import setuptools

USERNAME = 'beasteers'
NAME = 'rpiup'


setuptools.setup(
    name=NAME,
    version='0.0.6',
    description='',
    long_description=open('README.md').read().strip(),
    long_description_content_type='text/markdown',
    author='Bea Steers',
    author_email='bea.steers@gmail.com',
    url='https://github.com/{}/{}'.format(USERNAME, NAME),
    packages=setuptools.find_packages(),
    package_data={NAME: glob.glob(os.path.join(os.path.dirname(__file__), 'rpiup/boot-files/**/*'), recursive=True)},
    entry_points={'console_scripts': ['{name}={name}:cli'.format(name=NAME)]},
    install_requires=['fire'],
    extras_require={
        'monitor': ['flask', 'sqlitedict', 'PyYAML'],
    },
    license='BSD 3-Clause Clear License',
    keywords='raspberrypi raspberry pi os firstboot setup boot partition install iot fleet')
